@@ -1,7 +1,7 @@
 /* wmshutdown - dockapp to shutdown or reboot your machine
  *
  * Copyright 2001, 2002 Rafael V. Aroca <rafael@linuxqos.cjb.net>
- * Copyright 2014 Doug Torrance <dtorrance@monmouthcollege.edu>
+ * Copyright 2014 Doug Torrance <dtorrance@piedmont.edu>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,34 +19,40 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
+#include "wmshutdown.xpm"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#define GTK_RESPONSE_HALT   1
-#define GTK_RESPONSE_REBOOT 2
+enum {
+  GTK_RESPONSE_HALT,
+  GTK_RESPONSE_REBOOT,
+  GTK_RESPONSE_SUSPEND,
+  GTK_RESPONSE_HIBERNATE
+};
 
-#ifdef CONSOLEKIT
-#define HALT_METHOD      "Stop"
-#define REBOOT_METHOD    "Restart"
-#define DBUS_PATH        "/org/freedesktop/ConsoleKit/Manager"
-#define DBUS_INTERFACE   "org.freedesktop.ConsoleKit.Manager"
-#define DBUS_DESTINATION "org.freedesktop.ConsoleKit"
-#else
 #define HALT_METHOD      "PowerOff"
 #define REBOOT_METHOD    "Reboot"
+#define SUSPEND_METHOD   "Suspend"
+#define HIBERNATE_METHOD "Hibernate"
 #define DBUS_PATH        "/org/freedesktop/login1"
 #define DBUS_INTERFACE   "org.freedesktop.login1.Manager"
 #define DBUS_DESTINATION "org.freedesktop.login1"
-#endif
 
 static int showVersion;
+static gboolean show_suspend = FALSE;
+static gboolean show_hibernate = FALSE;
+
 GtkWidget *dialog = NULL;
 
 static GOptionEntry entries[] = {
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &showVersion,
 	  "Display version information", NULL },
+	{ "suspend", 's', 0, G_OPTION_ARG_NONE, &show_suspend,
+	  "Show 'suspend' button", NULL },
+	{ "hibernate", 'b', 0, G_OPTION_ARG_NONE, &show_hibernate,
+	  "Show 'hibernate' button", NULL },
 	{ NULL }
 };
 
@@ -58,7 +64,12 @@ GtkWidget *cria_dock(GtkWidget *mw, unsigned int s)
 {
 	GdkDisplay *display;
 	GtkWidget *foobox;
+	Display *d;
+	Window mainwin, iw, w, p, dummy1, *dummy2;
+	unsigned int dummy3;
+	XWMHints *wmHints;
 
+	(void) s;
 	display = gdk_display_get_default();
 	foobox = gtk_window_new(GTK_WINDOW_POPUP);
 
@@ -68,19 +79,17 @@ GtkWidget *cria_dock(GtkWidget *mw, unsigned int s)
 	gtk_widget_realize(mw);
 	gtk_widget_realize(foobox);
 
-	Display *d = GDK_DISPLAY_XDISPLAY(display);
-	Window mainwin = GDK_WINDOW_XID(gtk_widget_get_window(mw));
-	Window iw = GDK_WINDOW_XID(gtk_widget_get_window(foobox));
-	Window p, dummy1, *dummy2;
-	unsigned int dummy3;
+	d = GDK_DISPLAY_XDISPLAY(display);
+	mainwin = GDK_WINDOW_XID(gtk_widget_get_window(mw));
+	iw = GDK_WINDOW_XID(gtk_widget_get_window(foobox));
 	XQueryTree(d, mainwin, &dummy1, &p, &dummy2, &dummy3);
 	if (dummy2)
 		XFree(dummy2);
-	Window w = XCreateSimpleWindow(d, p, 0, 0, 1, 1, 0, 0, 0);
+	w = XCreateSimpleWindow(d, p, 0, 0, 1, 1, 0, 0, 0);
 	XReparentWindow(d, mainwin, w, 0, 0);
 	gtk_widget_show(mw);
 	gtk_widget_show(foobox);
-	XWMHints *wmHints = XGetWMHints(d, mainwin);
+	wmHints = XGetWMHints(d, mainwin);
 	if (!wmHints)
 		wmHints = XAllocWMHints();
 	wmHints->flags |= IconWindowHint;
@@ -107,15 +116,15 @@ void handle_click(GtkWidget *widget, gpointer data)
 
 	gchar *method = (gchar *)data;
 
+	fecha();
+
 	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);
 	message = g_dbus_message_new_method_call(
 			NULL,
 			DBUS_PATH,
 			DBUS_INTERFACE,
 			method);
-#ifndef CONSOLEKIT
 	g_dbus_message_set_body(message, g_variant_new("(b)", TRUE));
-#endif
 	g_dbus_message_set_destination(message, DBUS_DESTINATION);
 	reply = g_dbus_connection_send_message_with_reply_sync(
 		connection, message, 0, -1, NULL, NULL, &error);
@@ -147,9 +156,14 @@ void button_press(GtkWidget *widget, GdkEvent *event)
 	GtkWidget *label;
 	GtkWidget *halt_button;
 	GtkWidget *reboot_button;
+	GtkWidget *suspend_button;
+	GtkWidget *hibernate_button;
 	GtkWidget *cancel_button;
+	GdkEventButton  *bevent;
 
-	GdkEventButton  *bevent = (GdkEventButton *)event;
+	(void) widget;
+
+	bevent = (GdkEventButton *)event;
 	switch (bevent->button) {
 	case 1:
 		if (dialog != NULL)
@@ -162,23 +176,13 @@ void button_press(GtkWidget *widget, GdkEvent *event)
 
 		gtk_dialog_add_buttons(GTK_DIALOG(dialog),
 				       "Halt", GTK_RESPONSE_HALT,
-				       "Reboot", GTK_RESPONSE_REBOOT,
-				       "Cancel", GTK_RESPONSE_CANCEL, NULL);
+				       "Reboot", GTK_RESPONSE_REBOOT, NULL);
 		halt_button = gtk_dialog_get_widget_for_response(
 			GTK_DIALOG(dialog),
 			GTK_RESPONSE_HALT);
 		reboot_button = gtk_dialog_get_widget_for_response(
 			GTK_DIALOG(dialog),
 			GTK_RESPONSE_REBOOT);
-		cancel_button = gtk_dialog_get_widget_for_response(
-			GTK_DIALOG(dialog),
-			GTK_RESPONSE_CANCEL);
-
-		g_signal_connect(dialog, "destroy", G_CALLBACK(fecha), NULL);
-		g_signal_connect(cancel_button,
-				 "clicked",
-				 G_CALLBACK(fecha),
-				 (gpointer) dialog);
 		g_signal_connect(halt_button,
 				 "clicked",
 				 G_CALLBACK(handle_click),
@@ -187,6 +191,41 @@ void button_press(GtkWidget *widget, GdkEvent *event)
 				 "clicked",
 				 G_CALLBACK(handle_click),
 				 REBOOT_METHOD);
+
+		if (show_suspend) {
+		  gtk_dialog_add_buttons(GTK_DIALOG(dialog), "Suspend",
+					 GTK_RESPONSE_SUSPEND, NULL);
+		  suspend_button =
+		    gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog),
+						       GTK_RESPONSE_SUSPEND);
+		  g_signal_connect(suspend_button,
+				   "clicked",
+				   G_CALLBACK(handle_click),
+				   SUSPEND_METHOD);
+		}
+		if (show_hibernate) {
+		  gtk_dialog_add_buttons(GTK_DIALOG(dialog), "Hibernate",
+					 GTK_RESPONSE_HIBERNATE, NULL);
+		  hibernate_button =
+		    gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog),
+						       GTK_RESPONSE_HIBERNATE);
+		  g_signal_connect(hibernate_button,
+				   "clicked",
+				   G_CALLBACK(handle_click),
+				   HIBERNATE_METHOD);
+		}
+
+		gtk_dialog_add_buttons(GTK_DIALOG(dialog), "Cancel",
+				       GTK_RESPONSE_CANCEL, NULL);
+		cancel_button = gtk_dialog_get_widget_for_response(
+			GTK_DIALOG(dialog),
+			GTK_RESPONSE_CANCEL);
+		g_signal_connect(cancel_button,
+				 "clicked",
+				 G_CALLBACK(fecha),
+				 (gpointer) dialog);
+
+		g_signal_connect(dialog, "destroy", G_CALLBACK(fecha), NULL);
 		gtk_widget_show_all(dialog);
 		break;
 	default:
@@ -198,6 +237,7 @@ int main(int argc, char *argv[])
 {
 	GError *error = NULL;
 	GOptionContext *context;
+	GdkPixbuf *pixbuf;
 	GtkWidget *gtkiw;
 	GtkWidget *dockArea;
 	GtkWidget *pixmap;
@@ -218,7 +258,8 @@ int main(int argc, char *argv[])
 	gtkiw = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	dockArea = cria_dock(gtkiw, 47);
 
-	pixmap = gtk_image_new_from_file(DATADIR"/wmshutdown.xpm");
+	pixbuf = gdk_pixbuf_new_from_xpm_data(image_name);
+	pixmap = gtk_image_new_from_pixbuf(pixbuf);
 	gtk_widget_show(pixmap);
 	gtk_container_add(GTK_CONTAINER(dockArea), pixmap);
 
